__doc__ = """JPicture product module."""
__version__ = '0.1'  

from Globals import HTMLFile, MessageDialog
from OFS.Image import *

manage_addJPictureForm = HTMLFile('jpictureAdd', globals())

def manage_addJPicture(self, id, file, tn='', title='', description='', 
                       precondition='', content_type='', REQUEST=None):
    """Add a JPicture to a Folder"""

    id, title = cookId(id, title, file)

    self = self.this()
    self._setObject(id, JPicture(id, '','',  title, description, 
                    content_type, precondition))
    self._getOb(id).manage_upload(file)
  
    if tn is not '':
        self._getOb(id).manage_uploadTn(tn)

    if REQUEST is not None:
        return self.manage_main(self, REQUEST)


class JPicture(Image):
    """ JPicture: An expanded Image """

    meta_type = 'JPicture'

    manage_options = (
        {'label': 'Edit',        'action': 'manage_main'},
        {'label': 'Upload',      'action': 'manage_uploadForm'},
        {'label': 'Upload Thumb','action': 'manage_uploadThumbForm'},
        {'label': 'Properties',  'action': 'manage_propertiesForm'},
        {'label': 'View',        'action': ''}, #defaults to index_html
        {'label': 'View Thumb',  'action': 'viewThumb'},
        {'label': 'Security',    'action': 'manage_access'},
        )

    _properties=({'id':'title',          'type':'string'},
                 {'id':'description',    'type':'text'},
                 {'id':'content_type',   'type':'string'},
                 {'id':'size',           'type':'string'},
                 {'id':'height',         'type':'string'},
                 {'id':'width',          'type':'string'},
                 {'id':'tncontent_type', 'type':'string'},
                 {'id':'tnsize',         'type':'string'},
                 )
 
    __ac_permissions__ = (
        ('View management screens', 
            ('manage', 'manage_main', 'manage_uploadForm',
              'manage_uploadThumbForm')),
        ('Change JPictures',        
            ('manage_edit', 'manage_upload', 'manage_uploadTn')),
        ('View JPictures',
            ('index_html', 'tag', 'view_image_or_file', 'getSize',
             'getContentType', 'viewThumb', '')),
        )

    def __init__(self, id, file, tn='', title='', description = '',
                 content_type = '', tncontent_type='', precondition = ''):
        """ Initialize a new JPicture """
        self.__name__ = id
        self.title = title
        self.description = description
        self.precondition = precondition

        data, size = self._read_data(file)
        content_type=self._get_content_type(file, data, id, content_type)
        self.update_data(data, content_type, size)

        tndata, tnsize = self._read_data(tn)
        tncontent_type = self._get_content_type(tn, tndata, tncontent_type)
        self.update_tndata(tndata, tncontent_type, tnsize)

    manage_editForm = HTMLFile('jpictureEdit', globals())
    manage = manage_main = manage_editForm
    manage_uploadForm = HTMLFile('jpictureUpload', globals())
    manage_uploadThumbForm = HTMLFile('jpictureThumbUpload', globals())

    def manage_edit(self, title='', description='', REQUEST=None):
        """ Edit a JPicture """
        self.title = title
        self.description = description

        if REQUEST is not None:
            message = "Properties for %s changed." % self.id
            return self.manage_main(self, REQUEST, manage_tabs_message=message)
 
    def manage_uploadTn(self, tn='', REQUEST=None):
        """
        Replaces the current thumbnail of the JPicture object with tn 
        """
                                                                                
        tndata, tnsize = self._read_data(tn)

        tncontent_type = self._get_content_type(tn, tndata, self.__name__,
                                                self.tncontent_type)
        self.update_tndata(tndata, tncontent_type, tnsize)

        if REQUEST is not None: return MessageDialog(
            title = 'Success!',
            message = 'Your changes have been saved',
            action = 'manage_main')

    def update_tndata(self, tndata, tncontent_type=None, tnsize=None):
        if tncontent_type is not None: 
            self.tncontent_type = tncontent_type
        if tnsize is None: tnsize = len(tndata)
        self.tnsize = tnsize
        self.tndata = tndata

    def viewThumb(self, REQUEST, RESPONSE):
        """ Render the JPicture's Thumbnail """

        if self.tnsize is not 0:
            # HTTP If-Modified-Since header handling.
            header=REQUEST.get_header('If-Modified-Since', None)
            if header is not None:
                header=string.split(header, ';')[0]
                mod_since=int(DateTime(header).timeTime())
                if self._p_mtime:
                    last_mod = int(self._p_mtime)
                else:
                    last_mod = 0
                if last_mod > 0 and last_mod <= mod_since:
                    RESPONSE.setStatus(304)
                    return RESPONSE

            if self.precondition and hasattr(self,self.precondition):
                # Grab whatever precondition was defined and then
                # execute it.  The precondition will raise an exception
                # if something violates its terms.
                c=getattr(self,self.precondition)
                if hasattr(c,'isDocTemp') and c.isDocTemp:
                    c(REQUEST['PARENTS'][1],REQUEST)
                else:
                    c()
            RESPONSE.setHeader('Last-Modified', rfc1123_date(self._p_mtime))
            RESPONSE.setHeader('Content-Type', self.tncontent_type)
            RESPONSE.setHeader('Content-Length', self.tnsize)
                                                                                
            data=self.tndata
            if type(data) is type(''): return data
                                                                                
            while data is not None:
                RESPONSE.write(data.data)
                data=data.next
                                                                                
            return ''

        else:
            msg = 'No Thumbnail available'
            RESPONSE.setHeader('Content-Type', 'text/html')
            RESPONSE.setHeader('Content-Length', len(msg))
            RESPONSE.write(msg)

            return ''
